@@ -14,12 +14,18 @@ class SourceGeometrical(object):
                     spatial_type="Point",
                     angular_distribution = "Flat",
                     energy_distribution = "Single line",
+                    seed = 123456,
                  ):
 
         self.set_spatial_type_by_name(spatial_type)
         self.set_angular_distribution_by_name(angular_distribution)
         self.set_energy_distribution_by_name(energy_distribution)
 
+        self._seed = seed
+
+        if seed != 0:
+            numpy.random.seed(seed)
+
 
     #
     # spatial type
@@ -387,7 +393,12 @@ class SourceGeometrical(object):
                 rays[:,10] = self._energy_to_wavenumber(sampled_values)
             else:
                 rays[:,10] = self._wavelength_to_wavenumber(sampled_values)
-
+        elif self.energy_distribution == "Uniform":
+            sampled_values = self.__ph[0] + (self.__ph[1]-self.__ph[0]) * numpy.random.rand(N)
+            if self.__f_phot == 0:
+                rays[:,10] = self._energy_to_wavenumber(sampled_values)
+            else:
+                rays[:,10] = self._wavelength_to_wavenumber(sampled_values)
         elif self.energy_distribution == "Gaussian":
             sampled_values = numpy.random.normal(loc=self.__ph[0], scale=self.__ph[1], size=N)
             if self.__f_phot == 0:
@@ -502,6 +513,11 @@ class SourceGeometrical(object):
             u_norm[:,i] = uu
         return u / u_norm
 
+    def get_beam(self, N=5000, POL_DEG=1.0, POL_ANGLE=0.0, F_COHER=False):
+
+        rays =  self.calculate_rays(N=N,POL_DEG=POL_DEG,POL_ANGLE=POL_ANGLE,F_COHER=F_COHER)
+        return Beam.initialize_from_array(rays)
+
 if __name__ == "__main__":
     pass
 
