@@ -29,27 +29,31 @@ import scipy.constants as codata
 from scipy import interpolate
 
 from syned.storage_ring.magnetic_structures.undulator import Undulator
-from syned.storage_ring.electron_beam import ElectronBeam
+# from syned.storage_ring.electron_beam import ElectronBeam
+from shadow4.sources.s4_electron_beam import S4ElectronBeam
 
 from shadow4.beam.beam import Beam
 
-from shadow4.sources.undulator.source_undulator_factory import SourceUndulatorFactory
-from shadow4.sources.undulator.source_undulator_factory_srw import SourceUndulatorFactorySrw
-from shadow4.sources.undulator.source_undulator_factory_pysru import SourceUndulatorFactoryPysru
+from shadow4.sources.undulator.source_undulator_factory import calculate_undulator_emission # SourceUndulatorFactory
+from shadow4.sources.undulator.source_undulator_factory_srw import calculate_undulator_emission_SRW # SourceUndulatorFactorySrw
+from shadow4.sources.undulator.source_undulator_factory_pysru import calculate_undulator_emission_pySRU # SourceUndulatorFactoryPysru
 
 from shadow4.sources.undulator.s4_undulator import S4Undulator
 
 from syned.storage_ring.light_source import LightSource
 from shadow4.sources.s4_light_source import S4LightSource
 
+from shadow4.sources.source_geometrical.source_gaussian import SourceGaussian
+import scipy.constants as codata
+
 INTEGRATION_METHOD = 1 # 0=sum, 1=trapz
 
-class S4UndulatorLightSource(LightSource, S4LightSource):
+class S4UndulatorLightSource(S4LightSource):
 
-    def __init__(self, name="Undefined", electron_beam=None, undulator_magnetic_structure=None):
+    def __init__(self, name="Undefined", electron_beam=None, magnetic_structure=None):
         super().__init__(name,
-                         electron_beam=electron_beam if not electron_beam is None else ElectronBeam(),
-                         magnetic_structure=undulator_magnetic_structure if not undulator_magnetic_structure is None else S4Undulator())
+                         electron_beam=electron_beam if not electron_beam is None else S4ElectronBeam(),
+                         magnetic_structure=magnetic_structure if not magnetic_structure is None else S4Undulator())
 
 
         # results of calculations
@@ -58,103 +62,17 @@ class S4UndulatorLightSource(LightSource, S4LightSource):
         self.__result_photon_size_distribution = None
         self.__result_photon_size_sigma = None
 
-    def get_beam(self,user_unit_to_m=1.0,F_COHER=0,NRAYS=5000,SEED=123456):
-        return Beam.initialize_from_array(self.__calculate_rays(
-            user_unit_to_m=user_unit_to_m, F_COHER=F_COHER, NRAYS=NRAYS, SEED=SEED
-            ))
+    def get_beam(self, NRAYS=5000, SEED=123456):
+        user_unit_to_m = 1.0
+        F_COHER = 0
+        # use_gaussian_approximation = False
 
-    def info(self,debug=False):
-        syned_electron_beam = self.get_electron_beam()
-        undulator = self.get_magnetic_structure()
-
-        txt = ""
-
-        txt += "-----------------------------------------------------\n"
-
-        txt += "Input Electron parameters: \n"
-        txt += "        Electron energy: %f geV\n"%syned_electron_beam._energy_in_GeV
-        txt += "        Electron current: %f A\n"%syned_electron_beam._current
-        if undulator._FLAG_EMITTANCE:
-            sigmas = syned_electron_beam.get_sigmas_all()
-            txt += "        Electron sigmaX: %g [um]\n"%(1e6*sigmas[0])
-            txt += "        Electron sigmaZ: %g [um]\n"%(1e6*sigmas[2])
-            txt += "        Electron sigmaX': %f urad\n"%(1e6*sigmas[1])
-            txt += "        Electron sigmaZ': %f urad\n"%(1e6*sigmas[3])
-        txt += "Input Undulator parameters: \n"
-        txt += "        period: %f m\n"%undulator.period_length()
-        txt += "        number of periods: %d\n"%undulator.number_of_periods()
-        txt += "        K-value: %f\n"%undulator.K_vertical()
-
-        txt += "-----------------------------------------------------\n"
-
-        txt += "Lorentz factor (gamma): %f\n"%syned_electron_beam.gamma()
-        txt += "Electron velocity: %.12f c units\n"%(numpy.sqrt(1.0 - 1.0 / syned_electron_beam.gamma() ** 2))
-        txt += "Undulator length: %f m\n"%(undulator.period_length()*undulator.number_of_periods())
-        K_to_B = (2.0 * numpy.pi / undulator.period_length()) * codata.m_e * codata.c / codata.e
-
-        txt += "Undulator peak magnetic field: %f T\n"%(K_to_B*undulator.K_vertical())
-        txt += "Resonances: \n"
-        txt += "        harmonic number [n]                   %10d %10d %10d \n"%(1,3,5)
-        txt += "        wavelength [A]:                       %10.6f %10.6f %10.6f   \n"%(\
-                                                                1e10*undulator.resonance_wavelength(syned_electron_beam.gamma(),harmonic=1),
-                                                                1e10*undulator.resonance_wavelength(syned_electron_beam.gamma(),harmonic=3),
-                                                                1e10*undulator.resonance_wavelength(syned_electron_beam.gamma(),harmonic=5))
-        txt += "        energy [eV]   :                       %10.3f %10.3f %10.3f   \n"%(\
-                                                                undulator.resonance_energy(syned_electron_beam.gamma(),harmonic=1),
-                                                                undulator.resonance_energy(syned_electron_beam.gamma(),harmonic=3),
-                                                                undulator.resonance_energy(syned_electron_beam.gamma(),harmonic=5))
-        txt += "        frequency [Hz]:                       %10.3g %10.3g %10.3g   \n"%(\
-                                                                1e10*undulator.resonance_frequency(syned_electron_beam.gamma(),harmonic=1),
-                                                                1e10*undulator.resonance_frequency(syned_electron_beam.gamma(),harmonic=3),
-                                                                1e10*undulator.resonance_frequency(syned_electron_beam.gamma(),harmonic=5))
-        txt += "        central cone 'half' width [urad]:     %10.6f %10.6f %10.6f   \n"%(\
-                                                                1e6*undulator.gaussian_central_cone_aperture(syned_electron_beam.gamma(),1),
-                                                                1e6*undulator.gaussian_central_cone_aperture(syned_electron_beam.gamma(),3),
-                                                                1e6*undulator.gaussian_central_cone_aperture(syned_electron_beam.gamma(),5))
-        txt += "        first ring at [urad]:                 %10.6f %10.6f %10.6f   \n"%(\
-                                                                1e6*self.get_resonance_ring(1,1),
-                                                                1e6*self.get_resonance_ring(3,1),
-                                                                1e6*self.get_resonance_ring(5,1))
-
-        txt += "-----------------------------------------------------\n"
-        txt += "Grids: \n"
-        if undulator._NG_E == 1:
-            txt += "        photon energy %f eV\n"%(undulator._EMIN)
+        if self.get_magnetic_structure().use_gaussian_approximation():
+            return self.get_beam_in_gaussian_approximation(NRAYS=NRAYS, SEED=SEED)
         else:
-            txt += "        photon energy from %10.3f eV to %10.3f eV\n"%(undulator._EMIN,undulator._EMAX)
-        txt += "        number of points for the trajectory: %d\n"%(undulator._NG_J)
-        txt += "        number of energy points: %d\n"%(undulator._NG_E)
-        txt += "        maximum elevation angle: %f urad\n"%(1e6*undulator._MAXANGLE)
-        txt += "        number of angular elevation points: %d\n"%(undulator._NG_T)
-        txt += "        number of angular azimuthal points: %d\n"%(undulator._NG_P)
-        # txt += "        number of rays: %d\n"%(self.NRAYS)
-        # txt += "        random seed: %d\n"%(self.SEED)
-        txt += "-----------------------------------------------------\n"
-
-        txt += "calculation code: %s\n"%undulator.code_undul_phot
-        if self.__result_radiation is None:
-            txt += "radiation: NOT YET CALCULATED\n"
-        else:
-            txt += "radiation: CALCULATED\n"
-        txt += "Sampling: \n"
-        if undulator._FLAG_SIZE == 0:
-            flag = "point"
-        elif undulator._FLAG_SIZE == 1:
-            flag = "Gaussian"
-        elif undulator._FLAG_SIZE == 2:
-            flag = "Far field backpropagated"
-
-        txt += "        Photon source size sampling flag: %d (%s)\n"%(undulator._FLAG_SIZE,flag)
-        if undulator._FLAG_SIZE == 1:
-            if undulator.__result_photon_size_sigma is not None:
-                txt += "        Photon source size sigma (Gaussian): %6.3f um \n"%(1e6 * self.__result_photon_size_sigma)
-
-        txt += "-----------------------------------------------------\n"
-        return txt
-
-
-    def to_python_code(self):
-        return "# to be implemented..."
+            return Beam.initialize_from_array(self.__calculate_rays(
+                user_unit_to_m=user_unit_to_m, F_COHER=F_COHER, NRAYS=NRAYS, SEED=SEED
+                ))
 
 
     def get_resonance_ring(self,harmonic_number=1, ring_order=1):
@@ -166,36 +84,36 @@ class S4UndulatorLightSource(LightSource, S4LightSource):
         # take 3*sigma - _MAXANGLE is in RAD
         self._MAXANGLE = 3 * 0.69 * self.syned_undulator.gaussian_central_cone_aperture(self.syned_electron_beam.gamma(),harmonic_number)
 
-    def set_energy_monochromatic(self,emin):
-        """
-        Sets a single energy line for the source (monochromatic)
-        :param emin: the energy in eV
-        :return:
-        """
-        self._EMIN = emin
-        self._EMAX = emin
-        self._NG_E = 1
-
-
-    def set_energy_box(self,emin,emax,npoints=None):
-        """
-        Sets a box for photon energy distribution for the source
-        :param emin:  Photon energy scan from energy (in eV)
-        :param emax:  Photon energy scan to energy (in eV)
-        :param npoints:  Photon energy scan number of points (optinal, if not set no changes)
-        :return:
-        """
-        self._EMIN = emin
-        self._EMAX = emax
-        if npoints != None:
-            self._NG_E = npoints
-
-    def get_energy_box(self):
-        """
-        Gets the limits of photon energy distribution for the source
-        :return: emin,emax,number_of_points
-        """
-        return self._EMIN,self._EMAX,self._NG_E
+    # def set_energy_monochromatic(self,emin):
+    #     """
+    #     Sets a single energy line for the source (monochromatic)
+    #     :param emin: the energy in eV
+    #     :return:
+    #     """
+    #     self._EMIN = emin
+    #     self._EMAX = emin
+    #     self._NG_E = 1
+    #
+    #
+    # def set_energy_box(self,emin,emax,npoints=None):
+    #     """
+    #     Sets a box for photon energy distribution for the source
+    #     :param emin:  Photon energy scan from energy (in eV)
+    #     :param emax:  Photon energy scan to energy (in eV)
+    #     :param npoints:  Photon energy scan number of points (optinal, if not set no changes)
+    #     :return:
+    #     """
+    #     self._EMIN = emin
+    #     self._EMAX = emax
+    #     if npoints != None:
+    #         self._NG_E = npoints
+    #
+    # def get_energy_box(self):
+    #     """
+    #     Gets the limits of photon energy distribution for the source
+    #     :return: emin,emax,number_of_points
+    #     """
+    #     return self._EMIN,self._EMAX,self._NG_E
 
 
     def __calculate_radiation(self):
@@ -224,43 +142,49 @@ class S4UndulatorLightSource(LightSource, S4LightSource):
 
         # undul_phot
         if undulator.code_undul_phot == 'internal':
-            undul_phot_dict = SourceUndulatorFactory.calculate_undulator_emission(electron_energy= syned_electron_beam.energy(),
-                                                                                  electron_current= syned_electron_beam.current(),
-                                                                                  undulator_period= undulator.period_length(),
-                                                                                  undulator_nperiods= undulator.number_of_periods(),
-                                                                                  K         = undulator.K(),
-                                                                                  photon_energy= undulator._EMIN,
-                                                                                  EMAX      = undulator._EMAX,
-                                                                                  NG_E      = undulator._NG_E,
-                                                                                  MAXANGLE  = undulator._MAXANGLE,
-                                                                                  number_of_points= undulator._NG_T,
-                                                                                  NG_P      = undulator._NG_P,
-                                                                                  number_of_trajectory_points = undulator._NG_J)
+            undul_phot_dict = calculate_undulator_emission(
+                electron_energy= syned_electron_beam.energy(),
+                electron_current= syned_electron_beam.current(),
+                undulator_period= undulator.period_length(),
+                undulator_nperiods= undulator.number_of_periods(),
+                K         = undulator.K(),
+                photon_energy= undulator._EMIN,
+                EMAX      = undulator._EMAX,
+                NG_E      = undulator._NG_E,
+                MAXANGLE  = undulator._MAXANGLE,
+                number_of_points= undulator._NG_T,
+                NG_P      = undulator._NG_P,
+                number_of_trajectory_points = undulator._NG_J,
+                )
 
         elif undulator.code_undul_phot == 'pysru' or  undulator.code_undul_phot == 'pySRU':
-            undul_phot_dict = SourceUndulatorFactoryPysru.calculate_undulator_emission(electron_energy= syned_electron_beam.energy(),
-                                                                                       electron_current= syned_electron_beam.current(),
-                                                                                       undulator_period= undulator.period_length(),
-                                                                                       undulator_nperiods= undulator.number_of_periods(),
-                                                                                       K         = undulator.K(),
-                                                                                       photon_energy= undulator._EMIN,
-                                                                                       EMAX      = undulator._EMAX,
-                                                                                       NG_E      = undulator._NG_E,
-                                                                                       MAXANGLE  = undulator._MAXANGLE,
-                                                                                       number_of_points= undulator._NG_T,
-                                                                                       NG_P      = undulator._NG_P, )
+            undul_phot_dict = calculate_undulator_emission_pySRU(
+                electron_energy= syned_electron_beam.energy(),
+                electron_current= syned_electron_beam.current(),
+                undulator_period= undulator.period_length(),
+                undulator_nperiods= undulator.number_of_periods(),
+                K         = undulator.K(),
+                photon_energy= undulator._EMIN,
+                EMAX      = undulator._EMAX,
+                NG_E      = undulator._NG_E,
+                MAXANGLE  = undulator._MAXANGLE,
+                number_of_points= undulator._NG_T,
+                NG_P      = undulator._NG_P,
+                )
         elif undulator.code_undul_phot == 'srw' or  undulator.code_undul_phot == 'SRW':
-            undul_phot_dict = SourceUndulatorFactorySrw.calculate_undulator_emission(electron_energy= self.syned_electron_beam.energy(),
-                                                                                     electron_current= syned_electron_beam.current(),
-                                                                                     undulator_period= undulator.period_length(),
-                                                                                     undulator_nperiods= undulator.number_of_periods(),
-                                                                                     K         = undulator.K(),
-                                                                                     photon_energy= undulator._EMIN,
-                                                                                     EMAX      = undulator._EMAX,
-                                                                                     NG_E      = undulator._NG_E,
-                                                                                     MAXANGLE  = undulator._MAXANGLE,
-                                                                                     number_of_points= undulator._NG_T,
-                                                                                     NG_P      = undulator._NG_P, )
+            undul_phot_dict = calculate_undulator_emission_SRW(
+                electron_energy= syned_electron_beam.energy(),
+                electron_current= syned_electron_beam.current(),
+                undulator_period= undulator.period_length(),
+                undulator_nperiods= undulator.number_of_periods(),
+                K         = undulator.K(),
+                photon_energy= undulator._EMIN,
+                EMAX      = undulator._EMAX,
+                NG_E      = undulator._NG_E,
+                MAXANGLE  = undulator._MAXANGLE,
+                number_of_points= undulator._NG_T,
+                NG_P      = undulator._NG_P,
+                )
         else:
             raise Exception("Not implemented undul_phot code: "+undulator.code_undul_phot)
 
@@ -722,8 +646,8 @@ class S4UndulatorLightSource(LightSource, S4LightSource):
         from wofry.propagator.propagator import PropagationManager, PropagationElements, PropagationParameters
         from syned.beamline.beamline_element import BeamlineElement
         from shadow4.syned.element_coordinates import ElementCoordinates
-        from wofry.propagator.propagators1D.fresnel_zoom import FresnelZoom1D
-        from wofry.beamline.optical_elements.ideal_elements.screen import WOScreen1D
+        from wofryimpl.propagator.propagators1D.fresnel_zoom import FresnelZoom1D
+        from wofryimpl.beamline.optical_elements.ideal_elements.screen import WOScreen1D
 
 
         input_wavefront = GenericWavefront1D().initialize_wavefront_from_arrays(theta*distance,numpy.sqrt(radiation_flux)+0j)
@@ -829,6 +753,215 @@ class S4UndulatorLightSource(LightSource, S4LightSource):
 
         return sampled_photon_energy,sampled_theta,sampled_phi
 
+    def get_beam_in_gaussian_approximation(self, NRAYS=1000, SEED=12345):
+        emin, emax, npoints = self.get_magnetic_structure().get_energy_box()
+
+        if self.get_magnetic_structure().get_flag_emittance():
+            sigma_x, sigdi_x, sigma_z, sigdi_z = self.get_electron_beam().get_sigmas_all()
+            Sx, Sz, Spx, Spz = self.get_undulator_photon_beam_sizes_by_convolution(
+                sigma_x=sigma_x,
+                sigma_z=sigma_z,
+                sigdi_x=sigdi_x,
+                sigdi_z=sigdi_z,
+                undulator_E0=0.5*(emin+emax),
+                undulator_length=self.get_magnetic_structure().length(),
+            )
+            a = SourceGaussian.initialize_from_keywords(number_of_rays=NRAYS,
+                                                        sigmaX=Sx,
+                                                        sigmaY=0,
+                                                        sigmaZ=Sz,
+                                                        sigmaXprime=Spx,
+                                                        sigmaZprime=Spz,
+                                                        real_space_center=[0.0, 0.0, 0.0],
+                                                        direction_space_center=[0.0, 0.0])
+        else:
+            s, sp = self.get_undulator_photon_beam_sizes(
+                undulator_E0=0.5*(emin+emax),
+                undulator_length=self.get_magnetic_structure().length(),
+            )
+            a = SourceGaussian.initialize_from_keywords(number_of_rays=NRAYS,
+                                                        sigmaX=s,
+                                                        sigmaY=0,
+                                                        sigmaZ=s,
+                                                        sigmaXprime=sp,
+                                                        sigmaZprime=sp,
+                                                        real_space_center=[0.0, 0.0, 0.0],
+                                                        direction_space_center=[0.0, 0.0])
+
+
+        print(a.info())
+        beam = a.get_beam()
+        if emax != emin:
+            e = numpy.random.random(NRAYS) * (emax - emin) + emin
+            beam.set_photon_energy_eV(e)
+
+        return beam
+
+    @classmethod
+    def get_undulator_photon_beam_sizes(cls,
+                                        undulator_E0=15000.0,
+                                        undulator_length=4.0):
+        m2ev = codata.c * codata.h / codata.e
+        lambda1 = m2ev / undulator_E0
+        # calculate sizes of the photon undulator beam
+        # see formulas 25 & 30 in Elleaume (Onaki & Elleaume)
+        s_phot = 2.740/(4e0*numpy.pi)*numpy.sqrt(undulator_length*lambda1)
+        sp_phot = 0.69*numpy.sqrt(lambda1/undulator_length)
+        return s_phot, sp_phot
+
+    @classmethod
+    def get_undulator_photon_beam_sizes_by_convolution(cls,
+                                                       sigma_x=1e-4,
+                                                       sigma_z=1e-4,
+                                                       sigdi_x=1e-6,
+                                                       sigdi_z=1e-6,
+                                                       undulator_E0=15000.0,
+                                                       undulator_length=4.0):
+
+        user_unit_to_m = 1.0
+
+        s_phot, sp_phot = cls.get_undulator_photon_beam_sizes(undulator_E0=undulator_E0, undulator_length=undulator_length)
+
+        photon_h = numpy.sqrt(numpy.power(sigma_x*user_unit_to_m,2) + numpy.power(s_phot,2) )
+        photon_v = numpy.sqrt(numpy.power(sigma_z*user_unit_to_m,2) + numpy.power(s_phot,2) )
+        photon_hp = numpy.sqrt(numpy.power(sigdi_x,2) + numpy.power(sp_phot,2) )
+        photon_vp = numpy.sqrt(numpy.power(sigdi_z,2) + numpy.power(sp_phot,2) )
+
+        m2ev = codata.c * codata.h / codata.e
+        lambda1 = m2ev / undulator_E0
+        txt = ""
+        txt += "Photon single electron emission at wavelength %f A: \n"%(lambda1*1e10)
+        txt += "    sigma_u:      %g um \n"%(1e6*s_phot)
+        txt += "    sigma_uprime: %g urad \n" %(1e6*sp_phot)
+        txt += "Electron sizes: \n"
+        txt += "    sigma_x: %g um \n"%(1e6*sigma_x*user_unit_to_m)
+        txt += "    sigma_z: %g um \n" %(1e6*sigma_z*user_unit_to_m)
+        txt += "    sigma_x': %g urad \n"%(1e6*sigdi_x)
+        txt += "    sigma_z': %g urad \n" %(1e6*sigdi_z)
+        txt += "Photon source sizes (convolution): \n"
+        txt += "    Sigma_x: %g um \n"%(1e6*photon_h)
+        txt += "    Sigma_z: %g um \n" %(1e6*photon_v)
+        txt += "    Sigma_x': %g urad \n"%(1e6*photon_hp)
+        txt += "    Sigma_z': %g urad \n" %(1e6*photon_vp)
+
+        print(txt)
+
+        return (photon_h/user_unit_to_m, photon_v/user_unit_to_m, photon_hp, photon_vp)
+
+    ########################################
+    #
+    ########################################
+    def info(self,debug=False):
+        syned_electron_beam = self.get_electron_beam()
+        undulator = self.get_magnetic_structure()
+
+        txt = ""
+
+        txt += "-----------------------------------------------------\n"
+
+        txt += "Input Electron parameters: \n"
+        txt += "        Electron energy: %f geV\n"%syned_electron_beam._energy_in_GeV
+        txt += "        Electron current: %f A\n"%syned_electron_beam._current
+        if undulator._FLAG_EMITTANCE:
+            sigmas = syned_electron_beam.get_sigmas_all()
+            txt += "        Electron sigmaX: %g [um]\n"%(1e6*sigmas[0])
+            txt += "        Electron sigmaZ: %g [um]\n"%(1e6*sigmas[2])
+            txt += "        Electron sigmaX': %f urad\n"%(1e6*sigmas[1])
+            txt += "        Electron sigmaZ': %f urad\n"%(1e6*sigmas[3])
+        txt += "Input Undulator parameters: \n"
+        txt += "        period: %f m\n"%undulator.period_length()
+        txt += "        number of periods: %d\n"%undulator.number_of_periods()
+        txt += "        K-value: %f\n"%undulator.K_vertical()
+
+        txt += "-----------------------------------------------------\n"
+
+        txt += "Lorentz factor (gamma): %f\n"%syned_electron_beam.gamma()
+        txt += "Electron velocity: %.12f c units\n"%(numpy.sqrt(1.0 - 1.0 / syned_electron_beam.gamma() ** 2))
+        txt += "Undulator length: %f m\n"%(undulator.period_length()*undulator.number_of_periods())
+        K_to_B = (2.0 * numpy.pi / undulator.period_length()) * codata.m_e * codata.c / codata.e
+
+        txt += "Undulator peak magnetic field: %f T\n"%(K_to_B*undulator.K_vertical())
+        txt += "Resonances: \n"
+        txt += "        harmonic number [n]                   %10d %10d %10d \n"%(1,3,5)
+        txt += "        wavelength [A]:                       %10.6f %10.6f %10.6f   \n"%(\
+                                                                1e10*undulator.resonance_wavelength(syned_electron_beam.gamma(),harmonic=1),
+                                                                1e10*undulator.resonance_wavelength(syned_electron_beam.gamma(),harmonic=3),
+                                                                1e10*undulator.resonance_wavelength(syned_electron_beam.gamma(),harmonic=5))
+        txt += "        energy [eV]   :                       %10.3f %10.3f %10.3f   \n"%(\
+                                                                undulator.resonance_energy(syned_electron_beam.gamma(),harmonic=1),
+                                                                undulator.resonance_energy(syned_electron_beam.gamma(),harmonic=3),
+                                                                undulator.resonance_energy(syned_electron_beam.gamma(),harmonic=5))
+        txt += "        frequency [Hz]:                       %10.3g %10.3g %10.3g   \n"%(\
+                                                                1e10*undulator.resonance_frequency(syned_electron_beam.gamma(),harmonic=1),
+                                                                1e10*undulator.resonance_frequency(syned_electron_beam.gamma(),harmonic=3),
+                                                                1e10*undulator.resonance_frequency(syned_electron_beam.gamma(),harmonic=5))
+        txt += "        central cone 'half' width [urad]:     %10.6f %10.6f %10.6f   \n"%(\
+                                                                1e6*undulator.gaussian_central_cone_aperture(syned_electron_beam.gamma(),1),
+                                                                1e6*undulator.gaussian_central_cone_aperture(syned_electron_beam.gamma(),3),
+                                                                1e6*undulator.gaussian_central_cone_aperture(syned_electron_beam.gamma(),5))
+        txt += "        first ring at [urad]:                 %10.6f %10.6f %10.6f   \n"%(\
+                                                                1e6*self.get_resonance_ring(1,1),
+                                                                1e6*self.get_resonance_ring(3,1),
+                                                                1e6*self.get_resonance_ring(5,1))
+
+        txt += "-----------------------------------------------------\n"
+        txt += "Grids: \n"
+        if undulator._NG_E == 1:
+            txt += "        photon energy %f eV\n"%(undulator._EMIN)
+        else:
+            txt += "        photon energy from %10.3f eV to %10.3f eV\n"%(undulator._EMIN,undulator._EMAX)
+        txt += "        number of points for the trajectory: %d\n"%(undulator._NG_J)
+        txt += "        number of energy points: %d\n"%(undulator._NG_E)
+        txt += "        maximum elevation angle: %f urad\n"%(1e6*undulator._MAXANGLE)
+        txt += "        number of angular elevation points: %d\n"%(undulator._NG_T)
+        txt += "        number of angular azimuthal points: %d\n"%(undulator._NG_P)
+        # txt += "        number of rays: %d\n"%(self.NRAYS)
+        # txt += "        random seed: %d\n"%(self.SEED)
+        txt += "-----------------------------------------------------\n"
+
+        txt += "calculation code: %s\n"%undulator.code_undul_phot
+        if self.__result_radiation is None:
+            txt += "radiation: NOT YET CALCULATED\n"
+        else:
+            txt += "radiation: CALCULATED\n"
+        txt += "Sampling: \n"
+        if undulator._FLAG_SIZE == 0:
+            flag = "point"
+        elif undulator._FLAG_SIZE == 1:
+            flag = "Gaussian"
+        elif undulator._FLAG_SIZE == 2:
+            flag = "Far field backpropagated"
+
+        txt += "        Photon source size sampling flag: %d (%s)\n"%(undulator._FLAG_SIZE,flag)
+        if undulator._FLAG_SIZE == 1:
+            if undulator.__result_photon_size_sigma is not None:
+                txt += "        Photon source size sigma (Gaussian): %6.3f um \n"%(1e6 * self.__result_photon_size_sigma)
+
+        txt += "-----------------------------------------------------\n"
+        return txt
+
+
+
+    def to_python_code(self, data=None):
+        script = ''
+        try:
+            script += self.get_electron_beam().to_python_code()
+        except:
+            script += "\n\n#Error retrieving electron_beam code"
+
+        try:
+            script += self.get_magnetic_structure().to_python_code()
+        except:
+            script += "\n\n#Error retrieving magnetic structure code"
+
+
+        script += "\n\n\n# light source\nfrom shadow4.sources.undulator.s4_undulator_light_source import S4UndulatorLightSource"
+        script += "\nlight_source = S4UndulatorLightSource(name='%s', electron_beam=electron_beam, magnetic_structure=source)" % \
+                                                          (self.get_name())
+
+        script += "\n\n\n#beamline\nfrom shadow4.beamline.s4_beamline import S4Beamline"
+        script += "\nbeamline = S4Beamline(light_source=light_source)"
+        return script
 
 if __name__ == "__main__":
 
@@ -836,9 +969,14 @@ if __name__ == "__main__":
 
     do_plots = True
 
-    su = S4Undulator(K_vertical=0.25,period_length=0.032,number_of_periods=50)
+    su = S4Undulator(K_vertical=0.25,
+                     period_length=0.032,
+                     number_of_periods=50,
+                     code_undul_phot='SRW',
+                     use_gaussian_approximation=1,
+                     )
 
-    ebeam = ElectronBeam(energy_in_GeV=6.04,
+    ebeam = S4ElectronBeam(energy_in_GeV=6.04,
                  energy_spread = 0.0,
                  current = 0.2,
                  number_of_bunches = 400,
@@ -849,9 +987,10 @@ if __name__ == "__main__":
                  moment_yyp=0.0,
                  moment_ypyp=(4e-6)**2 )
 
-    ls = S4UndulatorLightSource(name="", electron_beam=ebeam, undulator_magnetic_structure=su)
-
-    print(ls.info())
+    ls = S4UndulatorLightSource(name="", electron_beam=ebeam, magnetic_structure=su)
 
     beam =  ls.get_beam()
 
+    print(ls.info())
+    print(ls.to_python_code())
+
